@@ -1,56 +1,40 @@
-# méthodes mixtes
-#----------------
-#' Operators to sum or substract POSIXctp from POSIXct or POSIXcti
-#'
-#' @name ops.posixctpi
-#' @aliases Ops.POSIXcti Ops.POSIXctp +,POSIXct,POSIXctp-method Ops.POSIXst
 setMethod ('+', c('POSIXct', 'POSIXctp'),
 	function (e1, e2) {
-		if (length (e1) > length(e2)) e2 <- e2[rep(1:length(e2), length.out=length(e1))]
-		if (length (e2) > length(e1)) e1 <- rep(e1, length.out=length(e2))
+		if (length (e1) > length(e2))
+			e2 <- e2[rep(1:length(e2), length.out=length(e1))]
+		if (length (e2) > length(e1))
+			e1 <- rep(e1, length.out=length(e2))
+
 		e3 <- as.POSIXlt (e1)
 		effective.unit <- sapply (FUN=switch, as.character (unit(e2) ),
 			year = 'year', month = 'mon', day = 'mday',
+			week = 'week',
 			hour = 'hour', minute = 'min', second = 'sec',
 			stop ('e2 is a corrupted POSIXctd') )
 		for (u in unique (effective.unit) ) {
 			index <- effective.unit == u
-			e3[[u]][index] <- e3[[u]][index] + duration (e2)[index]
-		}
-		return (as.POSIXct (e3) )
-	})
-#' @rdname ops.posixctpi
-#' @aliases +,POSIXctp,POSIXct-method
-setMethod ('+', c('POSIXctp', 'POSIXct'), function (e1, e2) e2 + e1)
-#' @rdname ops.posixctpi
-#' @aliases -,POSIXct,POSIXctp-method
-setMethod ('-', c('POSIXct', 'POSIXctp'),
-	function (e1, e2) {
-		e3 <- as.POSIXlt (e1)
-		effective.unit <- sapply (FUN=switch, as.character (unit(e2) ),
-			year = 'year', month = 'mon', day = 'mday',
-			hour = 'hour', minute = 'min', second = 'sec',
-			stop ('e2 is a corrupted POSIXctp') )
-		for (u in unique (effective.unit) ) {
-			index <- effective.unit == u
-			e3[[u]][index] <- e3[[u]][index] - duration (e2)[index]
+			if( u == 'week' ) {
+				e3[['mday']][index] <-
+					e3[['mday']][index] + 7*duration (e2)[index]
+			} else {
+				e3[[u]][index] <-
+					e3[[u]][index] + duration (e2)[index]
+			}
 		}
 		return (as.POSIXct (e3) )
 	})
 
-#' @rdname ops.posixctpi
-#' @aliases +,POSIXcti,POSIXctp-method
+setMethod ('+', c('POSIXctp', 'POSIXct'), function (e1, e2) e2 + e1)
+setMethod ('-', c('POSIXct', 'POSIXctp'), function (e1, e2) e1 + (-1 * e2))
+
 setMethod ('+', c('POSIXcti', 'POSIXctp'),
 	function (e1, e2) return (POSIXcti(start(e1) + e2, end(e1) + e2)))
-#' @rdname ops.posixctpi
-#' @aliases +,POSIXctp,POSIXcti-method
+
 setMethod ('+', c('POSIXctp', 'POSIXcti'), function (e1, e2) e2 + e1)
-#' @rdname ops.posixctpi
-#' @aliases -,POSIXcti,POSIXctp-method
+
 setMethod ('-', c('POSIXcti', 'POSIXctp'),
 	function (e1, e2) return (POSIXcti(start(e1) - e2, end(e1) - e2)))
-#' @rdname ops.posixctpi
-#' @aliases *,numeric,POSIXctp-method
+
 setMethod ('*', c('numeric', 'POSIXctp'),
 	function (e1, e2)
 	{
@@ -60,18 +44,37 @@ setMethod ('*', c('numeric', 'POSIXctp'),
 			unit <- rep (unit, length.out=length(duration))
 		return (POSIXctp(duration=duration, unit=unit))
 	} )
-#' @rdname ops.posixctpi
-#' @aliases +,POSIXctp,POSIXctp-method
+
+setMethod ('*', c('POSIXctp', 'numeric'), function (e1, e2) e2*e1 )
+
 setMethod ('+', c('POSIXctp', 'POSIXctp'),
 	function (e1, e2)
 	{
-		if (unit (e1) != unit(e2))
-			stop ("'e1' and 'e2' must have the same unit.")
-		unit <- as.character(unit(e2))
-		return (POSIXctp(duration=duration(e1) + duration(e2), unit=unit))
+	if (length (e1) > length(e2))
+		e2 <- e2[rep(1:length(e2), length.out=length(e1))]
+	if (length (e2) > length(e1))
+		e1 <- rep(e1, length.out=length(e2))
+
+	# first we try to convert the POSIXctp with the bigger unit to
+	# the unit of the other (if needed !)
+	
+	if(any( unit(e1) > unit(e2) ))
+		  unit(e1[unit(e1) > unit(e2)]) <- unit(e2)[unit(e1) > unit(e2)]
+
+	if(any( unit(e2) > unit(e1) )) 
+		unit(e2[unit(e2) > unit(e1)]) <- unit(e1)[unit(e2) > unit(e1)]
+
+	# then if two elements haven't the same unit, it means one is not
+	# comparable to the other thus the result is NA. Otherwise, durations
+	# are added
+
+	POSIXctp(
+		ifelse(unit(e1) != unit(e2), NA,
+		       duration(e1) + duration(e2)),
+		 unit(e1))
 	} )
-#' @rdname ops.posixctpi
-#' @aliases -,POSIXst,POSIXst-method
+setMethod ('-', c('POSIXctp', 'POSIXctp'), function(e1, e2) e1 + (-1*e2))
+
 setMethod ('-', c('POSIXst', 'POSIXst'),
 	function (e1, e2) {
 		if( unit(e1) != unit(e2) )
@@ -87,11 +90,9 @@ setMethod ('-', c('POSIXst', 'POSIXst'),
 
 		POSIXctp(e1@subtime - e2@subtime, rep(unit(e1), length(e1)))
 	} )
-#' @rdname ops.posixctpi
-#' @aliases +,POSIXctp,POSIXst-method
+
 setMethod ('+', c('POSIXctp', 'POSIXst'), function (e1, e2) e2 + e1)
-#' @rdname ops.posixctpi
-#' @aliases +,POSIXst,POSIXctp-method
+
 setMethod ('+', c('POSIXst', 'POSIXctp'),
 	function (e1, e2) {
 		if( unit(e1) != unit(e2) )
@@ -129,8 +130,7 @@ setMethod ('+', c('POSIXst', 'POSIXctp'),
 		if( any(sup) ) e3[sup] <- e3[sup] %% (val.max+1) + val.min
 		POSIXst(e3, uc, oc)
 	} )
-#' @rdname ops.posixctpi
-#' @aliases -,POSIXst,POSIXctp-method
+
 setMethod ('-', c('POSIXst', 'POSIXctp'),
 	function (e1, e2) {
 		if( unit(e1) != unit(e2) )
